
/*
 A.nnotate checks to see if there is a function defined with this name (annotate_note_editor)
 If so, then instead of showing the native note editor, it instantiates an
 annotate_note_editor with an interface object, the note editor peer, as its argument.
 The peer contains the methods for the replacement editor to communicate back
 to the annotate system.
 Similarly for annotate_note_display



 The skeleton functions are:

function annotate_note_editor(p) {
	this.peer = p;

	this.editNew = function(selectedText) {
		// called when the user has selected some text to start a new note
	}

	this.edit = function(note) {
		// called when the user has clicked the edit button on an existing note
		// N.B. if you implement the note display too, then it should use its peer to
		// tell annotate to edit the note, and annotate will then call this edit function.
		// I.e. don't short-circuit the editing - annotate needs to know what is being edited
	}

	// the peer object supplied when the constructor is called contains the functions:

	// peer.cancelNote();
	// this cancels note editing and clears any selection on the page

	// peer.saveAndShow(o);
	// o is an object with fields
	//  o.notetext    the text of the note
	//  o.subject     the text in the page that the note is attached to
	//  o.tags        any tags to be associated with the note
	//  o.color       the color to use for the highlight in the page
	//  o.linkTo      (optional) URL target for notes about a link
	//  o.linkTitle   (optional) Readable title of link

	// this saves and displays a new note. If annotate_note_display is defined, then its
	// display methods will be called to actually show the note. Again, don't short-circuit
	// the display because annotate needs to assign an id, store it on the server etc.
}




function annotate_note_display(p) {
	this.peer = p;

	function addNote(note) {
		// called when a new note has been written and saved
	}

	function removeAllNotes() {
		// only applies to multi-page documents that are displayed in an iframe where the
		// surrounding page should only display the notes for the active document page.
		// it is called when the page is changed in the document viewer   
	}

	function updateNote(note) {
		// called when an existing note has been edited
	}


	function highlightNote(noteid) {
		// called when the user clicks on the text in the page that the note is attached
		// to (and possibly other times, such as the a.nnotate note index or local index,
		// depending which parts are enabled).
	}

	// the peer object supplied when the constructor is called contains the functions:

	// peer.highlightNote(noteid);

	// peer.deleteNote(noteid);

	// peer.replyToNote(noteid);

	// peer.editNote(noteid);

	// peer.setHighlightColor(noteid);
	// sets the color of the target text background

	// peer.getHighlightIds(noteid);
	// returns an array of the ids of the elements in the page containing the target text.
	// This can be used to attach custom actions when the mouse enters the element for
	// example, as below.

*/





function annotate_note_editor(p) {
	var me = this;

	var peer = p;


	var noteTextarea = document.getElementById("note_textarea");
	var saveButton = document.getElementById("save_button");
	var cancelButton = document.getElementById("cancel_button");
	var urlInput = document.getElementById("note_link_to");
	var urlTitle = document.getElementById("note_link_title");
	var tagsInput = document.getElementById("tags_input");
	noteTextarea.style.backgroundColor = "#d0d0d0";


	saveButton.onclick = saveNote;
	cancelButton.onclick = cancelNote;


	var livenote = false;


	me.editNew = function(txt) {
		var n = {"subject":txt, "notetext":"", "tags":"", "linkTo":"", "linkTitle":""};
		me.edit(n);
	}


	me.edit = function(note) {
		noteTextarea.value = note.notetext;
		livenote = true;
		tagsInput.value = note.tags;
		urlInput.value = note.linkTo;
		urlTitle.value = note.linkTitle;
		noteTextarea.style.backgroundColor = "#ffffff";
	}


	function saveNote() {
		if (livenote) {
			var n = {};
			n.notetext = noteTextarea.value;
			n.linkTo = urlInput.value;
			n.linkTitle = urlTitle.value;
			n.tags = tagsInput.value;
			clearTextarea();
			n.color = "#ff8080";
			peer.saveAndShow(n);
		}
	}


	function cancelNote() {
		clearTextarea();
		peer.cancelNote();
	}

	function clearTextarea() {
		noteTextarea.value = "";
		urlInput.value = "";
		urlTitle.value = "";
		tagsInput.value = "";
		livenote = false;
		noteTextarea.style.backgroundColor = "#d0d0d0";
	}
}



function annotate_note_display(p) {
	var me = this;
	var peer = p;

	var notesdiv = document.getElementById("notes");
	var divs = [];
	var divsById = {};

	var ebut = document.getElementById("edit_button");
	var rbut = document.getElementById("reply_button");
	var dbut = document.getElementById("delete_button");
	ebut.onmousedown = editActive;
	rbut.onmousedown = replyActive;
	dbut.onmousedown = deleteActive;



	var activeid = "";

	me.highlightNote = function(noteid) {
		activeid = noteid;
		for (var i = 0; i < divs.length; i++) {
			divs[i].style.backgroundColor = "#ffffff";
		}
		var dv = divsById[noteid];
		dv.style.backgroundColor = "#ffffa0";
	}


	me.addNote = function(note) {
		var div = document.createElement("div");
		div.className = "note";
		notesdiv.appendChild(div);
		divsById[note.id] = div;
		divs[divs.length] = div;
		div.noteid = note.id;
		div.onmousedown = noteClicked;
		me.updateNote(note);

		var hover = new HoverActions(note);
		var ids = peer.getHighlightIds(note.id);
		for (var i = 0; i < ids.length; i++) {
			var elt = document.getElementById(ids[i]);
			elt.onmouseover = hover.entered;
		}
	}
	
	
	me.removeNote = function(note) {
		var div = divsById[note.id];
		delete divsById[note.id];
		div.parentNode.removeChild(div);
	}
	
	
	me.removeAllNotes = function() {
		while (notesdiv.hasChildNodes()) {
	        notesdiv.removeChild(notesdiv.lastChild);
	    }
		divs = [];
		divsById = {};
	}
	
	// This is called when a temporary ID gets replaced by a permanent one. The permanent 
	// one is useful if you want to create a link to a note.
	// It only apples to the case when the user creates a link to a note that has just been
	// created. After a page reload, all the notes have their definitive IDs.
	// The sequence of operations is:
	// a new note is created and assigned a temporary ID
	// addNote is called with the note data and the temporary ID (so the note can be displayed
	// immediately)
	// the data is sent to the server
	// the server assigns an ID to the note and tells annotate. A.nnotate calls gotFinalID
	// annotate calls updateNote
	// If an application does not need the final id for new notes, then it can use note.xid
	// instead of note.id and ignore gotFinalID.
	me.gotFinalID = function(oldid, newid) {
		divsById[newid] = divsById[oldid];
		divsById[newid].noteid = newid;
	}

	
	
	me.updateNote = function(note) {
		var div = divsById[note.id];
		if (!div) {
			alert("no div with id " + note.id);
		}
		empty(div);
		div.title = note.subject;
		if (note.linkTo) {
			var a = document.createElement("a");
			a.innerHTML = note.linkTitle;
			a.href = note.linkTo;
			var da = document.createElement("div");
			da.appendChild(a);
			div.appendChild(da);
		}
		if (note.tags) {
			var tspan = document.createElement("div");
			tspan.innerHTML = note.tags;
			tspan.className = "noteTags";
			div.appendChild(tspan);
		}

		var spn = document.createElement("span");
		spn.innerHTML = note.notetext;
		div.appendChild(spn);
		var att = document.createElement("div");
		div.appendChild(att);
		att.className = "noteMeta"
		att.innerHTML =   note.signed + " " + note.date;

		if (note.replies) {
			for (var i = 0; i < note.replies.length; i++) {
				var r = note.replies[i];
				var rdiv = document.createElement("div");
				rdiv.className = "reply";
				rdiv.innerHTML = r.signed +", " + r.date + ": " + r.notetext;
				div.appendChild(rdiv);
			}
		}
	}

	function empty(elt) {
		while (elt && elt.hasChildNodes()) {
	         elt.removeChild(elt.lastChild);
	    }
	}

	function HoverActions(n) {
		var note = n;

		this.entered = function() {
			var msgdiv = document.getElementById("message_area");
			empty(msgdiv);
			var p = document.createElement("p");
			p.innerHTML = "Highlight action example showing the text of " +
					"the last note the mouse visited:<br/><b> " + n.notetext + "</b>";
			msgdiv.appendChild(p);
		}
	}


	function noteClicked(e) {
		activeid = "";
		var dv = getTarget(e);
		if (dv.noteid) {
			activeid = dv.noteid;
		} else {
			var wk = dv.parentNode;
			while (wk) {
				if (wk.noteid) {
					activeid = wk.noteid;
					break;
				}
				wk = wk.parentNode;
			}
		}
		if (activeid != "") {
			peer.highlightNote(activeid);
		}
	}

	function editActive() {
		if (activeid != "") {
			peer.editNote(activeid);
		}
	}


	function replyActive() {
		if (activeid != "") {
			peer.replyToNote(activeid);
		}
	}



	function deleteActive() {
		if (activeid != "") {
			peer.deleteNote(activeid);
			if (divsById[activeid]) {
				divsById[activeid].style.display = "none";
			}
		}
	}



	function getTarget(x) {
	    x = (x || window.event);
	    return (x.target || x.srcElement);
    }

}


